@@ -1057,7 +1057,12 @@ menuproxy_build_cb (GObject * object, GAsyncResult * res, gpointer user_data)
 
 	/* If this wasn't cancelled, we should be good */
 	DbusmenuClient * client = DBUSMENU_CLIENT(user_data);
+	/* But let's check */
+	g_return_if_fail(client != NULL);
 	DbusmenuClientPrivate * priv = DBUSMENU_CLIENT_GET_PRIVATE(client);
+	g_return_if_fail(priv != NULL);
+
+
 	priv->menuproxy = proxy;
 
 	if (priv->menuproxy_cancel != NULL) {
@@ -1239,7 +1244,7 @@ menuproxy_signal_cb (GDBusProxy * proxy, gchar * sender, gchar * signal, GVarian
 	} else if (priv->root == NULL) {
 		/* Drop out here, all the rest of these really need to have a root
 		   node so we can just ignore them if there isn't one. */
-	} else if (g_strcmp0(signal, "ItemPropertiesUpdated") == 0) {
+	} else if (g_strcmp0(signal, "ItemsPropertiesUpdated") == 0) {
 		/* Remove before adding just incase there is a duplicate, against the
 		   rules, but we can handle it so let's do it. */
 		GVariantIter ritems;
@@ -1604,7 +1609,11 @@ about_to_show_cb (GObject * proxy, GAsyncResult * res, gpointer userdata)
 void
 dbusmenu_client_send_about_to_show(DbusmenuClient * client, gint id, void (*cb)(gpointer data), gpointer cb_data)
 {
+	g_return_if_fail(DBUSMENU_CLIENT(client));
+	g_return_if_fail(id > 0);
+
 	DbusmenuClientPrivate * priv = DBUSMENU_CLIENT_GET_PRIVATE(client);
+	g_return_if_fail(priv != NULL);
 
 	about_to_show_t * data = g_new0(about_to_show_t, 1);
 	data->client = client;
