@@ -28,7 +28,7 @@ License version 3 and version 2.1 along with this program.  If not, see
 
 #include "parser.h"
 #include "menuitem.h"
-#include "serializablemenuitem.h"
+#include "client.h"
 
 #define CACHED_MENUITEM  "dbusmenu-gtk-parser-cached-item"
 #define PARSER_DATA      "dbusmenu-gtk-parser-data"
@@ -270,6 +270,71 @@ new_menuitem (GtkWidget * widget)
 	return item;
 }
 
+static gboolean
+toggle_widget_visibility (GtkWidget * widget)
+{
+	gboolean vis = gtk_widget_get_visible (widget);
+	gtk_widget_set_visible (widget, !vis);
+	gtk_widget_set_visible (widget, vis);
+	g_object_unref (G_OBJECT (widget));
+	return FALSE;
+}
+
+static void
+watch_submenu(DbusmenuMenuitem * mi, GtkWidget * menu)
+{
+	g_return_if_fail(DBUSMENU_IS_MENUITEM(mi));
+	g_return_if_fail(GTK_IS_MENU_SHELL(menu));
+
+	ParserData *pdata = (ParserData *)g_object_get_data(G_OBJECT(mi), PARSER_DATA);
+
+	pdata->shell = menu;
+	g_signal_connect (G_OBJECT (menu),
+		          "child-added",
+		          G_CALLBACK (child_added_cb),
+		          mi);
+	g_signal_connect (G_OBJECT (menu),
+		          "child-removed",
+		          G_CALLBACK (child_removed_cb),
+		          mi);
+	g_object_add_weak_pointer(G_OBJECT (menu), (gpointer*)&pdata->shell);
+
+	/* Some apps (notably Eclipse RCP apps) don't fill contents of submenus
+	   until the menu is shown.  So we fake that by toggling the visibility of
+	   any submenus we come across.  Further, these apps need it done with a
+	   delay while they finish initializing, so we put the call in the idle
+	   queue. */
+	g_idle_add((GSourceFunc)toggle_widget_visibility,
+	           g_object_ref (G_OBJECT (menu)));
+}
+
+static void
+activate_toplevel_item (GtkWidget * item)
+{
+	/* Make sure that we have a menu item before we start calling
+	   functions that depend on it.  This should almost always be
+	   the case. */
+	if (!GTK_IS_MENU_ITEM(item)) {
+		return;
+	}
+
+	/* If the item is not opening a submenu we don't want to activate
+	   it as that'd cause an action.  Like opening a preferences dialog
+	   to the user.  That's not a good idea. */
+	if (gtk_menu_item_get_submenu(GTK_MENU_ITEM(item)) == NULL) {
+		return;
+	}
+
+	GtkWidget * shell = gtk_widget_get_parent (item);
+	if (!GTK_IS_MENU_BAR (shell)) {
+		return;
+	}
+
+	gtk_menu_shell_activate_item (GTK_MENU_SHELL (shell),
+	                              item,
+	                              TRUE);
+}
+
 static void
 parse_menu_structure_helper (GtkWidget * widget, RecurseContext * recurse)
 {
@@ -286,33 +351,14 @@ parse_menu_structure_helper (GtkWidget * widget, RecurseContext * recurse)
 		 * Note that this will not force menuitems in submenus to be updated as well.
 		 */
 		if (recurse->parent == NULL && GTK_IS_MENU_BAR(widget)) {
-			GList *children = gtk_container_get_children (GTK_CONTAINER (widget));
-			GList *iter;
-
-			for (iter = children; iter != NULL; iter = iter->next) {
-				gtk_menu_shell_activate_item (GTK_MENU_SHELL (widget),
-				                              iter->data,
-				                              TRUE);
-			}
-
-			g_list_free (children);
+			gtk_container_foreach (GTK_CONTAINER (widget),
+			                       (GtkCallback)activate_toplevel_item,
+			                       NULL);
 		}
 
 		if (recurse->parent == NULL) {
 			recurse->parent = new_menuitem(widget);
-
-			ParserData *pdata = (ParserData *)g_object_get_data(G_OBJECT(recurse->parent), PARSER_DATA);
-
-			pdata->shell = widget;
-			g_signal_connect (G_OBJECT (widget),
-				          "child-added",
-				          G_CALLBACK (child_added_cb),
-				          recurse->parent);
-			g_signal_connect (G_OBJECT (widget),
-				          "child-removed",
-				          G_CALLBACK (child_removed_cb),
-				          recurse->parent);
-			g_object_add_weak_pointer(G_OBJECT (widget), (gpointer*)&pdata->shell);
+			watch_submenu(recurse->parent, widget);
 		}
 
 		gtk_container_foreach (GTK_CONTAINER (widget),
@@ -448,13 +494,6 @@ sanitize_label (GtkLabel * label)
 static DbusmenuMenuitem *
 construct_dbusmenu_for_widget (GtkWidget * widget)
 {
-	/* If it's a subclass of our serializable menu item then we can
-	   use its own build function */
-	if (DBUSMENU_IS_GTK_SERIALIZABLE_MENU_ITEM(widget)) {
-		DbusmenuGtkSerializableMenuItem * smi = DBUSMENU_GTK_SERIALIZABLE_MENU_ITEM(widget);
-		return dbusmenu_gtk_serializable_menu_item_build_menuitem(smi);
-	}
-
   /* If it's a standard GTK Menu Item we need to do some of our own work */
   if (GTK_IS_MENU_ITEM (widget))
     {
@@ -561,16 +600,7 @@ construct_dbusmenu_for_widget (GtkWidget * widget)
           GtkWidget *submenu = gtk_menu_item_get_submenu(GTK_MENU_ITEM(widget));
           if (submenu)
             {
-              pdata->shell = submenu;
-              g_signal_connect (G_OBJECT (submenu),
-                                "child-added",
-                                G_CALLBACK (child_added_cb),
-                                mi);
-              g_signal_connect (G_OBJECT (submenu),
-                                "child-removed",
-                                G_CALLBACK (child_removed_cb),
-                                mi);
-              g_object_add_weak_pointer(G_OBJECT(submenu), (gpointer*)&pdata->shell);
+              watch_submenu(mi, submenu);
             }
 
           if (!g_object_get_data (G_OBJECT (widget), "gtk-empty-menu-item") && !GTK_IS_TEAROFF_MENU_ITEM (widget))
@@ -846,6 +876,9 @@ label_notify_cb (GtkWidget  *widget,
           g_idle_add ((GSourceFunc)recreate_menu_item_in_idle_cb, child);
         } 
     }
+
+  g_value_unset(&prop_value);
+  return;
 }
 
 static void
@@ -1077,6 +1110,7 @@ widget_notify_cb (GtkWidget  *widget,
 	  if (item != NULL) {
         GtkWidget * menu = GTK_WIDGET (g_value_get_object (&prop_value));
         parse_menu_structure_helper(menu, &recurse);
+        watch_submenu(item, menu);
       } else {
         /* Note: it would be really odd that we wouldn't have a cached
            item, but we should handle that appropriately. */
@@ -1106,6 +1140,10 @@ child_added_cb (GtkContainer *menu, GtkWidget *widget, gpointer data)
 	recurse.toplevel = gtk_widget_get_toplevel(GTK_WIDGET(menu));
 	recurse.parent = menuitem;
 
+	if (GTK_IS_MENU_BAR(menu)) {
+		activate_toplevel_item (widget);
+	}
+
 	parse_menu_structure_helper(widget, &recurse);
 }
 
